@@ -1,7 +1,7 @@
 /* saslint.h - internal SASL library definitions
  * Rob Siemborski
  * Tim Martin
- * $Id: saslint.h,v 1.48 2003/04/16 19:36:01 rjs3 Exp $
+ * $Id: saslint.h,v 1.52 2004/02/20 23:54:51 rjs3 Exp $
  */
 /* 
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
@@ -189,7 +189,7 @@ typedef struct mechanism
 {
     int version;
     int condition; /* set to SASL_NOUSER if no available users;
-		      set to SASL_CONTINUE if delayed plugn loading */
+		      set to SASL_CONTINUE if delayed plugin loading */
     char *plugname; /* for AUTHSOURCE tracking */
     const sasl_server_plug_t *plug;
     struct mechanism *next;
@@ -453,8 +453,6 @@ sasl_string_list_t *_sasl_server_mechs(void);
  */
 extern int sasl_config_init(const char *filename);
 extern const char *sasl_config_getstring(const char *key,const char *def);
-extern int sasl_config_getint(const char *key,int def);
-extern int sasl_config_getswitch(const char *key,int def);
 
 /* checkpw.c */
 #ifdef DO_SASL_CHECKAPOP
