@@ -1,6 +1,6 @@
 /* NTLM SASL plugin
  * Ken Murchison
- * $Id: ntlm.c,v 1.18 2003/11/03 18:25:25 ken3 Exp $
+ * $Id: ntlm.c,v 1.24 2004/03/09 18:02:30 rjs3 Exp $
  *
  * References:
  *   http://www.innovation.ch/java/ntlm.html
@@ -94,7 +94,7 @@
 
 /*****************************  Common Section  *****************************/
 
-static const char plugin_id[] = "$Id: ntlm.c,v 1.18 2003/11/03 18:25:25 ken3 Exp $";
+static const char plugin_id[] = "$Id: ntlm.c,v 1.24 2004/03/09 18:02:30 rjs3 Exp $";
 
 #ifdef WIN32
 static ssize_t writev (SOCKET fd, const struct iovec *iov, size_t iovcnt);
@@ -639,15 +639,15 @@ static void load_session_setup(unsigned char buf[], SMB_SessionSetup *setup)
 
     *p++ = setup->andx_command;
     *p++ = setup->andx_reserved;
-    *((uint16 *) p) = setup->andx_offset; p += 2;
-    *((uint16 *) p) = setup->max_buffer_size; p += 2;
-    *((uint16 *) p) = setup->max_mpx_count; p += 2;
-    *((uint16 *) p) = setup->vc_number; p += 2;
-    *((uint32 *) p) = setup->session_key; p += 4;
-    *((uint16 *) p) = setup->case_insensitive_passwd_len; p += 2;
-    *((uint16 *) p) = setup->case_sensitive_passwd_len; p += 2;
-    *((uint32 *) p) = setup->reserved; p += 4;
-    *((uint32 *) p) = setup->capabilities;
+    htois(p, setup->andx_offset); p += 2;
+    htois(p, setup->max_buffer_size); p += 2;
+    htois(p, setup->max_mpx_count); p += 2;
+    htois(p, setup->vc_number); p += 2;
+    htoil(p, setup->session_key); p += 4;
+    htois(p, setup->case_insensitive_passwd_len); p += 2;
+    htois(p, setup->case_sensitive_passwd_len); p += 2;
+    htoil(p, setup->reserved); p += 4;
+    htoil(p, setup->capabilities); p += 4;
 }
 
 static void unload_session_setup_resp(unsigned char buf[],
@@ -742,7 +742,7 @@ static int retry_read(SOCKET fd, char *buf0, unsigned nbyte)
 
 	nread += n;
 
-	if (nread >= (int) nbyte) return nread;
+	if (n >= (int) nbyte) return nread;
 
 	buf += n;
 	nbyte -= n;
@@ -790,6 +790,7 @@ static int smb_connect_server(const sasl_utils_t *utils, const char *client,
 #else
     int saved_errno;
 #endif
+    int niflags;
     char *port = "139";
     char hbuf[NI_MAXHOST], pbuf[NI_MAXSERV];
 
@@ -832,10 +833,19 @@ static int smb_connect_server(const sasl_utils_t *utils, const char *client,
 #endif
 	closesocket (s);
 	s = -1;
-	getnameinfo(r->ai_addr, r->ai_addrlen,
-		    hbuf, sizeof(hbuf), pbuf, sizeof(pbuf),
-		    NI_NUMERICHOST | NI_WITHSCOPEID | NI_NUMERICSERV);
-/* Can't use errno (and %m), as it doesn't contain the socket error on Windows */
+	niflags = (NI_NUMERICHOST | NI_NUMERICSERV);
+#ifdef NI_WITHSCOPEID
+	if (r->ai_family == AF_INET6)
+	    niflags |= NI_WITHSCOPEID;
+#endif
+	if (getnameinfo(r->ai_addr, r->ai_addrlen, hbuf, sizeof(hbuf),
+			pbuf, sizeof(pbuf), niflags) != 0) {
+	    strcpy(hbuf, "unknown");
+	    strcpy(pbuf, "unknown");
+	}
+
+        /* Can't use errno (and %m), as it doesn't contain
+         * the socket error on Windows */
 	error_str = _plug_get_error_message (utils, saved_errno);
 	utils->log(NULL, SASL_LOG_WARN, "NTLM: connect %s[%s]/%s: %s",
 		   ai->ai_canonname ? ai->ai_canonname : server,
@@ -845,8 +855,10 @@ static int smb_connect_server(const sasl_utils_t *utils, const char *client,
 	utils->free (error_str);
     }
     if (s < 0) {
-	getnameinfo(ai->ai_addr, ai->ai_addrlen, NULL, 0,
-			pbuf, sizeof(pbuf), NI_NUMERICSERV);
+	if (getnameinfo(ai->ai_addr, ai->ai_addrlen, NULL, 0,
+			pbuf, sizeof(pbuf), NI_NUMERICSERV) != 0) {
+		strcpy(pbuf, "unknown");
+	}
 	utils->log(NULL, SASL_LOG_ERR, "NTLM: couldn't connect to %s/%s",
 		   ai->ai_canonname ? ai->ai_canonname : server, pbuf);
 	freeaddrinfo(ai);
@@ -1541,6 +1553,9 @@ static int ntlm_server_mech_step2(server_context_t *text,
 	password->len = pass_len;
 	strncpy(password->data, auxprop_values[0].values[0], pass_len + 1);
 
+	/* erase the plaintext password */
+	sparams->utils->prop_erase(sparams->propctx, password_request[0]);
+
 	/* calculate our own response(s) and compare with client's */
 	result = SASL_OK;
 	if (nt_resp && (nt_resp_len > NTLM_RESP_LENGTH)) {
